﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AirFighter
{
    public partial class FormMap : Form
    {
        private DrawingAirFighter _airFighter;
        private AbstractMap _abstractMap;

        public FormMap()
        {
            InitializeComponent();
            _abstractMap = new SimpleMap();
        }


        private void SetData(DrawingAirFighter airFighter)
        {
            toolStripStatusLabelSpeed.Text = $"Скорость: {_airFighter.AirFighter.Speed}";
            toolStripStatusLabelWeight.Text = $"Вес: {_airFighter.AirFighter.Weight}";
            toolStripStatusLabelBodyColor.Text = $"Цвет: { _airFighter.AirFighter.BodyColor.Name}";

            pictureBox.Image = _abstractMap.CreateMap(pictureBox.Width, pictureBox.Height, new DrawingObjectAirFighter(_airFighter));
        }

        private void CreateButton_Click(object sender, EventArgs e)
        {
            Random rnd = new();

            _airFighter = new DrawingAirFighter(rnd.Next(100, 300), rnd.Next(1000, 2000),
                             Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)));

            SetData(_airFighter);
        }

        private void CreateModernButton_Click(object sender, EventArgs e)
        {
            Random rnd = new();

            _airFighter = new DrawingModernAirFighter(rnd.Next(100, 300), rnd.Next(1000, 2000),
                Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)),
                Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)),
                Convert.ToBoolean(rnd.Next(0, 2)), Convert.ToBoolean(rnd.Next(0, 2)));


            SetData(_airFighter);
        }

        private void ButtonMove_Click(object sender, EventArgs e)
        {
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "UpButton":
                    dir = Direction.Up;
                    break;
                case "DownButton":
                    dir = Direction.Down;
                    break;
                case "LeftButton":
                    dir = Direction.Left;
                    break;
                case "RightButton":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _abstractMap?.MoveObject(dir);

        }

        private void PictureBox_Resize(object sender, EventArgs e)
        {
            _airFighter?.ChangeBorders(pictureBox.Width, pictureBox.Height);
        }


        public void Draw()
        {
            if (pictureBox.Width == 0 || pictureBox.Height == 0) return;
            Bitmap bmp = new(pictureBox.Width, pictureBox.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _airFighter?.DrawTransport(gr);
            pictureBox.Image = bmp;
        }

        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (comboBoxSelectorMap.Text)
            {
                case "простая карта":
                    _abstractMap = new SimpleMap();
                    break;
                case "моя карта":
                    _abstractMap = new MyMap();
                    break;
            }
        }


    }
}
