﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirFighter
{
    internal class MapWithSetCarsGeneric<T, U>
 where T : class, IDrawingObject
 where U : AbstractMap
    {
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Размер занимаемого объектом места (ширина)
        /// </summary>
        private readonly int _placeSizeWidth = 210;
        /// <summary>
        /// Размер занимаемого объектом места (высота)
        /// </summary>
        private readonly int _placeSizeHeight = 170;
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly SetCarsGeneric<T> _setCars;
        /// <summary>
        /// Карта
        /// </summary>
        private readonly U _map;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="picWidth"></param>
        /// <param name="picHeight"></param>
        /// <param name="map"></param>
        public MapWithSetCarsGeneric(int picWidth, int picHeight, U map)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _setCars = new SetCarsGeneric<T>(width * height);
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _map = map;
        }
        /// <summary>
        /// Перегрузка оператора сложения
        /// </summary>
        /// <param name="map"></param>
        /// <param name="car"></param>
        /// <returns></returns>
        public static int operator +(MapWithSetCarsGeneric<T, U> map, T car)
        {
            return map._setCars.Insert(car);
        }
        /// <summary>
        /// Перегрузка оператора вычитания
        /// </summary>
        /// <param name="map"></param>
        /// <param name="position"></param>
        /// <returns></returns>
        public static T operator -(MapWithSetCarsGeneric<T, U> map, int
       position)
        {
            return map._setCars.Remove(position);
        }
        /// <summary>
        /// Вывод всего набора объектов
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowSet()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawCars(gr);
            return bmp;
        }
        /// <summary>
        /// Просмотр объекта на карте
        /// </summary>
        /// <returns></returns>
        public Bitmap ShowOnMap()
        {
            Shaking();

            foreach(var car in _setCars.GetCars())
            {
                return _map.CreateMap(_pictureWidth, _pictureHeight, car);
            }
            return new(_pictureWidth, _pictureHeight);
        }
        /// <summary>
        /// Перемещение объекта по крате
        /// </summary>
        /// <param name="direction"></param>
        /// <returns></returns>
        public Bitmap MoveObject(Direction direction)
        {
            if (_map != null)
            {
                return _map.MoveObject(direction);
            }
            return new(_pictureWidth, _pictureHeight);
        }

        public string GetData(char separatorType, char separatorData)
        {
            string data = $"{_map.GetType().Name}{separatorType}";
            foreach (var car in _setCars.GetCars())
            {
                data += $"{car.GetInfo()}{separatorData}";
            }
            return data;
        }
        public void LoadData(string[] records)
        {
            foreach (var rec in records)
            {
                _setCars.Insert(DrawingObjectAirFighter.Create(rec) as T);
            }
        }

        private void Shaking()
        {
            int j = _setCars.Count - 1;
            for (int i = 0; i < _setCars.Count; i++)
            {
                if (_setCars[i] == null)
                {
                    for (; j > i; j--)
                    {
                        var car = _setCars[j];
                        if (car != null)
                        {
                            _setCars.Insert(car, i);
                            _setCars.Remove(j);
                            break;
                        }
                    }
                    if (j <= i)
                    {
                        return;
                    }
                }
            }
        }
        /// <summary>
        /// Метод отрисовки фона
        /// </summary>
        /// <param name="g"></param>
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            Brush brownBrush = new SolidBrush(Color.FromArgb(255, 211, 136, 84));
            Brush greyBrush = new SolidBrush(Color.FromArgb(255, 160, 160, 160));

            Point[] angar =
            {
                new(0, _pictureHeight ),
                new(0, _pictureHeight / 4 ),
                new(_pictureWidth / 2 , 9),
                new(_pictureWidth, _pictureHeight / 4 ),
                new(_pictureWidth, _pictureHeight ),
            };

            g.FillPolygon(brownBrush, angar);
            g.FillRectangle(greyBrush, _pictureWidth / 6, (_pictureHeight * 5) / 12, (_pictureWidth * 2) / 3, (_pictureHeight * 7) / 12);

            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
                {//линия рамзетки места
                    g.DrawLine(pen, i * _placeSizeWidth, j * _placeSizeHeight, i *
                   _placeSizeWidth + _placeSizeWidth / 2, j * _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i * _placeSizeWidth,
               (_pictureHeight / _placeSizeHeight) * _placeSizeHeight);
            }
        }
        /// <summary>
        /// Метод прорисовки объектов
        /// </summary>
        /// <param name="g"></param>
        private void DrawCars(Graphics g)
        {
            int width = _pictureWidth / _placeSizeWidth;
            int height = _pictureHeight / _placeSizeHeight;

            int i = 0;

            foreach(var car in _setCars.GetCars())
            {
                int x = i % width;
                int y = i / width;

                car.SetObject((width - x - 1) * _placeSizeWidth, (height - y - 1) * _placeSizeHeight, _pictureWidth, _pictureHeight);
                car.DrawningObject(g);

                i++;
            }
        }
    }
}
